package com.sun.java.swing.plaf.motif;

import javax.swing.border.EmptyBorder;
import javax.swing.plaf.ComponentUI;
import javax.swing.JScrollBar;
import java.beans.PropertyChangeEvent;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import java.beans.PropertyChangeListener;
import javax.swing.border.CompoundBorder;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicScrollPaneUI;

public class MotifScrollPaneUI extends BasicScrollPaneUI
{
    private static final Border vsbMarginBorderR;
    private static final Border vsbMarginBorderL;
    private static final Border hsbMarginBorder;
    private CompoundBorder vsbBorder;
    private CompoundBorder hsbBorder;
    private PropertyChangeListener propertyChangeHandler;
    
    public MotifScrollPaneUI() {
    }
    
    @Override
    protected void installListeners(JScrollPane c) {
        super.installListeners(c);
        c.addPropertyChangeListener(this.propertyChangeHandler = this.createPropertyChangeHandler());
    }
    
    @Override
    protected void uninstallListeners(JComponent c) {
        super.uninstallListeners(c);
        c.removePropertyChangeListener(this.propertyChangeHandler);
    }
    
    private PropertyChangeListener createPropertyChangeHandler() {
        return new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                if (propertyChangeEvent.getPropertyName().equals("componentOrientation")) {
                    JScrollPane scrollPane = (JScrollPane)propertyChangeEvent.getSource();
                    JScrollBar verticalScrollBar = scrollPane.getVerticalScrollBar();
                    if (verticalScrollBar != null && MotifScrollPaneUI.this.vsbBorder != null && verticalScrollBar.getBorder() == MotifScrollPaneUI.this.vsbBorder) {
                        if (MotifGraphicsUtils.isLeftToRight(scrollPane)) {
                            MotifScrollPaneUI.this.vsbBorder = new CompoundBorder(MotifScrollPaneUI.vsbMarginBorderR, MotifScrollPaneUI.this.vsbBorder.getInsideBorder());
                        }
                        else {
                            MotifScrollPaneUI.this.vsbBorder = new CompoundBorder(MotifScrollPaneUI.vsbMarginBorderL, MotifScrollPaneUI.this.vsbBorder.getInsideBorder());
                        }
                        verticalScrollBar.setBorder(MotifScrollPaneUI.this.vsbBorder);
                    }
                }
            }
        };
    }
    
    @Override
    protected void installDefaults(JScrollPane scrollpane) {
        super.installDefaults(scrollpane);
        JScrollBar verticalScrollBar = scrollpane.getVerticalScrollBar();
        if (verticalScrollBar != null) {
            if (MotifGraphicsUtils.isLeftToRight(scrollpane)) {
                this.vsbBorder = new CompoundBorder(MotifScrollPaneUI.vsbMarginBorderR, verticalScrollBar.getBorder());
            }
            else {
                this.vsbBorder = new CompoundBorder(MotifScrollPaneUI.vsbMarginBorderL, verticalScrollBar.getBorder());
            }
            verticalScrollBar.setBorder(this.vsbBorder);
        }
        JScrollBar horizontalScrollBar = scrollpane.getHorizontalScrollBar();
        if (horizontalScrollBar != null) {
            horizontalScrollBar.setBorder(this.hsbBorder = new CompoundBorder(MotifScrollPaneUI.hsbMarginBorder, horizontalScrollBar.getBorder()));
        }
    }
    
    @Override
    protected void uninstallDefaults(JScrollPane c) {
        super.uninstallDefaults(c);
        JScrollBar verticalScrollBar = this.scrollpane.getVerticalScrollBar();
        if (verticalScrollBar != null) {
            if (verticalScrollBar.getBorder() == this.vsbBorder) {
                verticalScrollBar.setBorder(null);
            }
            this.vsbBorder = null;
        }
        JScrollBar horizontalScrollBar = this.scrollpane.getHorizontalScrollBar();
        if (horizontalScrollBar != null) {
            if (horizontalScrollBar.getBorder() == this.hsbBorder) {
                horizontalScrollBar.setBorder(null);
            }
            this.hsbBorder = null;
        }
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new MotifScrollPaneUI();
    }
    
    static {
        vsbMarginBorderR = new EmptyBorder(0, 4, 0, 0);
        vsbMarginBorderL = new EmptyBorder(0, 0, 0, 4);
        hsbMarginBorder = new EmptyBorder(4, 0, 0, 0);
    }
}
